@@ -6,7 +6,7 @@ Dependency-less LDAP filter parser for Python
 :author: Thomas Calmant
 :copyright: Copyright 2013, isandlaTech
 :license: GPLv3
-:version: 0.3.1
+:version: 0.4
 :status: Alpha
 
 ..
@@ -33,8 +33,8 @@ import inspect
 # ------------------------------------------------------------------------------
 
 # Module version
-__version_info__ = (0, 3, 1)
-__version__ = ".".join(map(str, __version_info__))
+__version_info__ = (0, 4, 0)
+__version__ = ".".join(str(x) for x in __version_info__)
 
 # Documentation strings format
 __docformat__ = "restructuredtext en"
@@ -129,9 +129,9 @@ class LDAPFilter(object):
 
     def append(self, ldap_filter):
         """
-        Appends a filter or a criteria to this filter
+        Appends a filter or a criterion to this filter
 
-        :param ldap_filter: An LDAP filter or criteria
+        :param ldap_filter: An LDAP filter or criterion
         :raise TypeError: If the parameter is not of a known type
         :raise ValueError: If the more than one filter is associated to a
                            NOT operator
@@ -153,24 +153,20 @@ class LDAPFilter(object):
         :param properties: A dictionary of properties
         :return: True if the properties matches this filter, else False
         """
-        result = False
+        # Use a generator, and declare it outside of the method call
+        # => seems to be quite a speed up trick
+        generator = (criterion.matches(properties)
+                     for criterion in self.subfilters)
 
-        for criteria in self.subfilters:
-            if not criteria.matches(properties):
-                if self.operator == AND:
-                    # A criteria doesn't match in an "AND" test : short cut
-                    result = False
-                    break
+        # Extract "if" from loops and use built-in methods
+        if self.operator == OR:
+            result = any(generator)
 
-            else:
-                result = True
-                if self.operator == OR:
-                    # At least one match in a "OR" test : short cut
-                    break
-
-        if self.operator == NOT:
-            # Revert result
-            return not result
+        else:
+            result = all(generator)
+            if self.operator == NOT:
+                # Revert result
+                return not result
 
         return result
 
@@ -211,17 +207,17 @@ class LDAPFilter(object):
 
 class LDAPCriteria(object):
     """
-    Represents an LDAP criteria
+    Represents an LDAP criterion
     """
     def __init__(self, name, value, comparator):
         """
-        Sets up the criteria
+        Sets up the criterion
 
         :raise ValueError: If one of the parameters is empty
         """
         if not name or not value or not comparator:
             # Refuse empty values
-            raise ValueError("Invalid criteria parameter ({0}, {1}, {2})"\
+            raise ValueError("Invalid criterion parameter ({0}, {1}, {2})"\
                              .format(name, value, comparator))
 
         if not inspect.isroutine(comparator):
@@ -281,22 +277,23 @@ class LDAPCriteria(object):
 
     def matches(self, properties):
         """
-        Tests if the given criteria matches this LDAP criteria
+        Tests if the given criterion matches this LDAP criterion
 
         :param properties: A dictionary of properties
-        :return: True if the properties matches this criteria, else False
+        :return: True if the properties matches this criterion, else False
         """
-        if self.name not in properties:
-            # Property is not even is the property
-            return False
+        try:
+            # Use the comparator
+            return self.comparator(self.value, properties[self.name])
 
-        # Use the comparator
-        return self.comparator(self.value, properties[self.name])
+        except KeyError:
+            # Criterion key is not in the properties
+            return False
 
 
     def normalize(self):
         """
-        Returns this criteria
+        Returns this criterion
         """
         return self
 
@@ -306,10 +303,10 @@ def comparator2str(comparator):
     """
     Converts an operator method to a string
     """
-    if comparator == _comparator_approximate:
+    if comparator in (_comparator_approximate, _comparator_approximate_star):
         return "~="
 
-    elif comparator == _comparator_eq:
+    elif comparator in (_comparator_eq, _comparator_star):
         return "="
 
     elif comparator == _comparator_le:
@@ -411,24 +408,43 @@ def unescape_LDAP(ldap_string):
 
 # ------------------------------------------------------------------------------
 
+ITERABLES = (list, tuple, set)
+""" The types that are considered iterable in comparators """
+
+def _comparator_presence(_, tested_value):
+    """
+    Tests a filter which simply a joker, i.e. a value presence test
+    """
+    # The filter value is a joker : simple presence test
+    if tested_value is None:
+        return False
+
+    elif hasattr(tested_value, "__len__"):
+        # Refuse empty values
+        return len(tested_value) != 0
+
+    # Presence validated
+    return True
+
+
 def _comparator_star(filter_value, tested_value):
     """
-    Tests a filter with a joker
+    Tests a filter containing a joker
     """
-    if filter_value == '*':
-        # The filter value is a joker : simple presence test
-        if tested_value is None:
-            return False
+    if isinstance(tested_value, ITERABLES):
+        for value in tested_value:
+            if _star_comparison(filter_value, value):
+                return True
 
-        elif hasattr(tested_value, "__len__"):
-            # Refuse empty values
-            return len(tested_value) != 0
+    else:
+        return _star_comparison(filter_value, tested_value)
 
-        else:
-            # Presence validated
-            return True
 
-    elif not is_string(tested_value):
+def _star_comparison(filter_value, tested_value):
+    """
+    Tests a filter containing a joker
+    """
+    if not is_string(tested_value):
         # Unhandled value type...
         return False
 
@@ -468,40 +484,27 @@ def _comparator_eq(filter_value, tested_value):
     """
     Tests if the filter value is equal to the tested value
     """
-    if isinstance(tested_value, (list, tuple)):
-        # Special case : lists
-        if '*' in filter_value:
-            # Special case : jokers (stars) in the filter
-            for value in tested_value:
-                if _comparator_star(filter_value, value):
-                    # One match
-                    return True
-
+    if isinstance(tested_value, ITERABLES):
         # Convert the list items to strings
         for value in tested_value:
-
             # Try with the string conversion
             if not is_string(value):
-                value_str = repr(value)
-            else:
-                value_str = value
+                value = repr(value)
 
-            if filter_value == value_str:
+            if filter_value == value:
                 # Match !
                 return True
 
-        # Value not found
-        return False
-
-    elif '*' in filter_value:
-        # Special case : jokers (stars) in the filter
-        return _comparator_star(filter_value, tested_value)
-
     # Standard comparison
-    if not is_string(tested_value):
+    elif not is_string(tested_value):
+        # String vs string representation
         return filter_value == repr(tested_value)
 
-    return filter_value == tested_value
+    else:
+        # String vs string
+        return filter_value == tested_value
+
+    return False
 
 
 def _comparator_approximate(filter_value, tested_value):
@@ -531,6 +534,34 @@ def _comparator_approximate(filter_value, tested_value):
         or _comparator_eq(lower_filter_value, tested_value)
 
 
+def _comparator_approximate_star(filter_value, tested_value):
+    """
+    Tests if the filter value, which contains a joker, is nearly equal to the
+    tested value.
+
+    If the tested value is a string or an array of string, it compares their
+    lower case forms
+    """
+    lower_filter_value = filter_value.lower()
+
+    if is_string(tested_value):
+        # Lower case comparison
+        return _comparator_star(lower_filter_value, tested_value.lower())
+
+    elif hasattr(tested_value, '__iter__'):
+        # Extract a list of strings
+        new_tested = [value.lower() for value in tested_value \
+                      if is_string(value)]
+
+        if _comparator_star(lower_filter_value, new_tested):
+            # Value found in the strings
+            return True
+
+    # Compare the raw values
+    return _comparator_star(filter_value, tested_value) \
+        or _comparator_star(lower_filter_value, tested_value)
+
+
 def _comparator_le(filter_value, tested_value):
     """
     Tests if the filter value is greater than the tested value
@@ -717,9 +748,9 @@ def _skip_spaces(string, idx):
     return -1
 
 
-def _parse_LDAP_criteria(ldap_filter, startidx=0, endidx= -1):
+def _parse_LDAP_criteria(ldap_filter, startidx=0, endidx=-1):
     """
-    Parses an LDAP sub filter (criteria)
+    Parses an LDAP sub filter (criterion)
 
     :param ldap_filter: An LDAP filter string
     :param startidx: Sub-filter start index
@@ -787,9 +818,22 @@ def _parse_LDAP_criteria(ldap_filter, startidx=0, endidx= -1):
     # Skip spaces
     i = _skip_spaces(ldap_filter, i)
 
-    # Extract the value name
+    # Extract the value
     value = ldap_filter[i:endidx].strip()
 
+    # Use the appropriate comparator if a joker is found in the filter value
+    if value == '*':
+        # Presence comparator
+        comparator = _comparator_presence
+
+    elif '*' in value:
+        # Joker
+        if comparator == _comparator_eq:
+            comparator = _comparator_star
+
+        elif comparator == _comparator_approximate:
+            comparator = _comparator_approximate_star
+
     return LDAPCriteria(unescape_LDAP(attribute_name), unescape_LDAP(value),
                         comparator)
 
@@ -843,19 +887,19 @@ def _parse_LDAP(ldap_filter):
                 # Ending filter : store it in its parent
 
                 if len(subfilter_stack) != 0:
-                    # Criteria finished
+                    # criterion finished
                     startidx = subfilter_stack.pop()
-                    criteria = _parse_LDAP_criteria(ldap_filter, startidx, idx)
+                    criterion = _parse_LDAP_criteria(ldap_filter, startidx, idx)
 
                     if len(stack) != 0:
                         top = stack.pop()
-                        top.append(criteria)
+                        top.append(criterion)
                         stack.append(top)
                     else:
-                        # No parent : filter contains only one criteria
+                        # No parent : filter contains only one criterion
                         # Make a parent to stay homogeneous
                         root = LDAPFilter(AND)
-                        root.append(criteria)
+                        root.append(criterion)
 
                 elif len(stack) != 0:
                     # Sub filter finished
@@ -887,7 +931,7 @@ def _parse_LDAP(ldap_filter):
 
     # No root : invalid content
     if root is None:
-        raise ValueError("Invalid filter string")
+        raise ValueError("Invalid filter string: {0}".format(ldap_filter))
 
     # Return the root of the filter
     return root.normalize()
