@@ -14,7 +14,7 @@ when using remote services on the local host only.
 :author: Thomas Calmant
 :copyright: Copyright 2013, isandlaTech
 :license: GPLv3
-:version: 0.1
+:version: 0.2
 :status: Alpha
 
 ..
@@ -36,8 +36,8 @@ when using remote services on the local host only.
 """
 
 # Module version
-__version_info__ = (0, 1, 1)
-__version__ = ".".join(map(str, __version_info__))
+__version_info__ = (0, 2, 0)
+__version__ = ".".join(str(x) for x in __version_info__)
 
 # Documentation strings format
 __docformat__ = "restructuredtext en"
@@ -47,8 +47,8 @@ __docformat__ = "restructuredtext en"
 # Pelix utilities
 from pelix.utilities import to_bytes, to_str
 
-# Remote Services constants
-import pelix.remote.beans
+# Remote services
+import pelix.remote
 
 # iPOPO decorators
 from pelix.ipopo.decorators import ComponentFactory, Requires, Provides, \
@@ -61,15 +61,16 @@ import os
 import select
 import socket
 import struct
-import sys
 import threading
 
-if sys.version_info[0] < 3:
-    import httplib
-
-else:
+try:
+    # Python 3
     import http.client as httplib
 
+except ImportError:
+    # Python 2
+    import httplib
+
 # ------------------------------------------------------------------------------
 
 _logger = logging.getLogger(__name__)
@@ -275,7 +276,7 @@ def close_multicast_socket(sock, address):
 
 # ------------------------------------------------------------------------------
 
-@ComponentFactory("pelix-remote-discovery-multicast-factory")
+@ComponentFactory(pelix.remote.FACTORY_DISCOVERY_MULTICAST)
 @Provides(pelix.remote.SERVICE_ENDPOINT_LISTENER)
 @Requires("_dispatcher", pelix.remote.SERVICE_DISPATCHER)
 @Requires('_access', pelix.remote.SERVICE_DISPATCHER_SERVLET)
@@ -379,27 +380,6 @@ class MulticastDiscovery(object):
         self.__send_packet(data)
 
 
-    def _send_discovered(self, sender):
-        """
-        Sends a "discovered" packet to the one that sent a "discovery" packet
-        
-        :param sender: An (address, port) tuple
-        """
-        # Get the dispatcher servlet access
-        access = self._access.get_access()
-
-        # Make the discovery packet content
-        data = {"event": "discovered",  # "Discovered" packet
-                "sender": self._fw_uid,  # Framework UID
-                "access": {"port": access[0],  # Access to the dispatcher
-                           "path": access[1]}  # servlet
-                }
-
-        # Send a JSON request
-        data = json.dumps(data)
-        self.__send_packet(data, sender)
-
-
     def endpoint_added(self, endpoint):
         """
         A new service is exported
@@ -447,16 +427,9 @@ class MulticastDiscovery(object):
         event = data['event']
         if event == "discovery":
             # Discovery request
-            self._send_discovered(sender)
-
-        elif event == "discovered":
-            # Answer to a discovery request
             access = data['access']
-            endpoints = self.grab_endpoints(sender[0], access['port'],
-                                            access['path'])
-            if endpoints:
-                for endpoint in endpoints:
-                    self._register_endpoint(sender[0], endpoint)
+            self._access.send_discovered(sender[0], access['port'],
+                                         access['path'])
 
         elif event in ('add', 'update', 'remove'):
             # End point event
@@ -484,7 +457,7 @@ class MulticastDiscovery(object):
             endpoint = self.grab_endpoint(sender[0], access['port'],
                                           access['path'], endpoint_uid)
             if endpoint is not None:
-                self._register_endpoint(sender[0], endpoint)
+                self._access.register_endpoint(sender[0], endpoint)
 
         elif event == 'remove':
             # Remove it
@@ -493,66 +466,10 @@ class MulticastDiscovery(object):
         elif event == 'update':
             # Update it
             new_properties = data['new_properties']
-            self.__filter_properties(framework_uid, new_properties)
+            self._access.filter_properties(framework_uid, new_properties)
             self._registry.update(endpoint_uid, new_properties)
 
 
-    def __filter_properties(self, framework_uid, properties):
-        """
-        Replaces in-place export properties by import ones
-        
-        :param framework_uid: The UID of the framework exporting the service
-        :param properties: End point properties
-        :return: The filtered dictionary.
-        """
-        # Add the "imported" property
-        properties[pelix.remote.PROP_IMPORTED] = True
-
-        # Replace the "exported configs"
-        if pelix.remote.PROP_EXPORTED_CONFIGS in properties:
-            properties[pelix.remote.PROP_IMPORTED_CONFIGS] = \
-                                properties[pelix.remote.PROP_EXPORTED_CONFIGS]
-
-        # Clear export properties
-        for name in (pelix.remote.PROP_EXPORTED_CONFIGS,
-                     pelix.remote.PROP_EXPORTED_INTERFACES):
-            if name in properties:
-                del properties[name]
-
-        # Add the framework UID to the properties
-        properties[pelix.remote.PROP_FRAMEWORK_UID] = framework_uid
-
-        return properties
-
-
-    def _register_endpoint(self, host_address, endpoint_dict):
-        """
-        Registers a new end point in the registry
-        
-        :param host_address: Address of the service exporter
-        :param endpoint_dict: An end point description dictionary (result of 
-                              a request to the dispatcher servlet)
-        """
-        # Get the UID of the framework exporting the service
-        framework = endpoint_dict['sender']
-
-        # Filter properties
-        properties = self.__filter_properties(framework,
-                                              endpoint_dict['properties'])
-
-        # Format the URL
-        url = endpoint_dict['url'].format(server=host_address)
-
-        # Create the end point object
-        endpoint = pelix.remote.beans.ImportEndpoint(endpoint_dict['uid'], \
-                                 framework, endpoint_dict['kind'],
-                                 endpoint_dict['name'], url,
-                                 endpoint_dict['specifications'], properties)
-
-        # Register it
-        self._registry.add(endpoint)
-
-
     def grab_endpoint(self, host, port, path, uid):
         """
         Retrieves the description of the end point with the given UID at the
@@ -576,28 +493,6 @@ class MulticastDiscovery(object):
         return self.__grab_data(host, port, request_path)
 
 
-    def grab_endpoints(self, host, port, path):
-        """
-        Retrieves all end points available in the dispatcher servlet at the
-        given path.
-        Returns the result of the dispatcher servlet (list of dictionaries), or
-        None in case of error.
-        Does not register nor converts the end points.
-
-        :param host: Dispatcher host address
-        :param port: Dispatcher HTTP service port
-        :param path: Path to the dispatcher servlet
-        :return: A list of dictionaries or None
-        """
-        # Setup the request URI
-        if path[-1] == '/':
-            path = path[:-1]
-
-        request_path = "{0}/endpoints".format(path)
-
-        return self.__grab_data(host, port, request_path)
-
-
     def __grab_data(self, host, port, path):
         """
         Sends a HTTP request to the server at (host, port), on the given path.
