@@ -9,7 +9,7 @@ A patched version of jsonrpclib will be released soon.
 :author: Thomas Calmant
 :copyright: Copyright 2013, isandlaTech
 :license: GPLv3
-:version: 0.1
+:version: 0.1.2
 :status: Alpha
 
 ..
@@ -34,8 +34,8 @@ A patched version of jsonrpclib will be released soon.
 """
 
 # Module version
-__version_info__ = (0, 1, 1)
-__version__ = ".".join(map(str, __version_info__))
+__version_info__ = (0, 1, 2)
+__version__ = ".".join(str(x) for x in __version_info__)
 
 # Documentation strings format
 __docformat__ = "restructuredtext en"
@@ -48,7 +48,7 @@ from pelix.ipopo.decorators import ComponentFactory, Requires, Validate, \
 
 # Pelix constants
 import pelix.http
-import pelix.remote
+import pelix.remote.beans
 from pelix.utilities import to_str
 
 # Standard library
@@ -120,7 +120,7 @@ class _JsonRpcServlet(SimpleJSONRPCDispatcher):
 
 # ------------------------------------------------------------------------------
 
-@ComponentFactory("pelix-jsonrpc-exporter-factory")
+@ComponentFactory(pelix.remote.FACTORY_TRANSPORT_JSONRPC_EXPORTER)
 @Requires('_dispatcher', pelix.remote.SERVICE_DISPATCHER)
 @Requires('_http', pelix.http.HTTP_SERVICE)
 @Property('_path', pelix.http.HTTP_SERVLET_PATH, '/JSON-RPC')
@@ -362,7 +362,7 @@ class JsonRpcServiceExporter(object):
         self._http.unregister(None, self._servlet)
 
         # Remove all exports
-        for reference in self.__registrations.keys():
+        for reference in list(self.__registrations.keys()):
             self._unexport_service(reference)
 
         # Clean up the storage
@@ -402,7 +402,7 @@ class _ServiceCallProxy(object):
         return getattr(proxy, "{0}.{1}".format(self.__name, name))
 
 
-@ComponentFactory("pelix-jsonrpc-importer-factory")
+@ComponentFactory(pelix.remote.FACTORY_TRANSPORT_JSONRPC_IMPORTER)
 @Provides(pelix.remote.SERVICE_ENDPOINT_LISTENER)
 @Property('_kind', 'endpoints.kind', 'jsonrpc')
 @Property('_listener_flag', pelix.remote.PROP_LISTEN_IMPORTED, True)
