@@ -9,7 +9,7 @@ available.
 :author: Thomas Calmant
 :copyright: Copyright 2013, isandlaTech
 :license: GPLv3
-:version: 0.1.1
+:version: 0.1.3
 :status: Alpha
 
 ..
@@ -31,8 +31,8 @@ available.
 """
 
 # Module version
-__version_info__ = (0, 1, 2)
-__version__ = ".".join(map(str, __version_info__))
+__version_info__ = (0, 1, 3)
+__version__ = ".".join(str(x) for x in __version_info__)
 
 # Documentation strings format
 __docformat__ = "restructuredtext en"
@@ -99,8 +99,7 @@ class InteractiveShell(object):
         self.search_shell()
 
         # Register as a service listener
-        ldap_filter = '({0}={1})'.format(pelix.OBJECTCLASS, SHELL_SERVICE_SPEC)
-        self._context.add_service_listener(self, ldap_filter)
+        self._context.add_service_listener(self, None, SHELL_SERVICE_SPEC)
 
 
     def loop_input(self, on_quit=None):
