@@ -8,7 +8,7 @@ Provides the basic command parsing and execution support to make a Pelix shell.
 :author: Thomas Calmant
 :copyright: Copyright 2013, isandlaTech
 :license: GPLv3
-:version: 0.2
+:version: 0.2.3
 :status: Alpha
 
 ..
@@ -30,8 +30,8 @@ Provides the basic command parsing and execution support to make a Pelix shell.
 """
 
 # Module version
-__version_info__ = (0, 2, 2)
-__version__ = ".".join(map(str, __version_info__))
+__version_info__ = (0, 2, 3)
+__version__ = ".".join(str(x) for x in __version_info__)
 
 # Documentation strings format
 __docformat__ = "restructuredtext en"
@@ -43,16 +43,25 @@ from pelix.shell import SHELL_SERVICE_SPEC, SHELL_COMMAND_SPEC, \
     SHELL_UTILS_SERVICE_SPEC
 
 # Pelix modules
-from pelix.utilities import to_str
+from pelix.utilities import to_str, to_bytes
+import pelix.constants as constants
 import pelix.framework as pelix
 
 # Standard library
-import io
+import inspect
 import logging
 import os
 import shlex
 import sys
 import traceback
+import threading
+
+# Before Python 3, input() was raw_input()
+if sys.version_info[0] < 3:
+    safe_input = raw_input
+
+else:
+    safe_input = input
 
 # ------------------------------------------------------------------------------
 
@@ -138,47 +147,72 @@ def _split_ns_command(cmd_token):
 
 # ------------------------------------------------------------------------------
 
-class IOHandler(io.RawIOBase):
+class IOHandler(object):
     """
     Handles I/O operations between the command handler and the client
     It automatically converts the given data to bytes in Python 3.
     """
-    def __init__(self, instream, out_stream, encoding='UTF-8'):
+    def __init__(self, in_stream, out_stream, encoding='UTF-8'):
         """
         Sets up the printer
-        
-        :param instream: Input stream
+
+        :param in_stream: Input stream
         :param out_stream: Output stream
         :param encoding: Output encoding
         """
-        self.input = instream
+        self.input = in_stream
         self.output = out_stream
         self.encoding = encoding
 
-        # Set up class methods
-        if sys.version_info[0] == 3:
-            self.read = self._read_python3
-            self.write = self._write_python3
+        # Standard behavior
+        self.flush = self.output.flush
+        self.write = self.output.write
 
-        else:
-            self.read = self.input.read
-            self.write = self.output.write
+        # Specific behavior
+        if sys.version_info[0] >= 3:
+            if 'b' in getattr(out_stream, 'mode', ''):
+                # Bytes conversion
+                self.write = self._write_bytes
+            else:
+                # Strings accepted
+                self.write = self._write_str
+
+        # Very specific
+        if in_stream is sys.stdin:
+            # Warning: conflicts with the console
+            self.prompt = safe_input
 
 
-    def _read_python3(self, n):
+    def prompt(self, prompt=None):
         """
-        Reads the input stream
-        
-        :param n: Maximum bytes to read
-        :return: The result of ``self.input.read()``
+        Reads a line written by the user
+
+        :param prompt: An optional prompt message
+        :return: The read line, after a conversion to str
         """
-        return to_str(self.input.read(n), self.encoding)
+        if prompt:
+            # Print the prompt
+            self.write(prompt)
+            self.output.flush()
+
+        # Read the line
+        return to_str(self.input.readline())
 
 
-    def _write_python3(self, data):
+    def _write_bytes(self, data):
         """
         Converts the given data then writes it
-        
+
+        :param data: Data to be written
+        :return: The result of ``self.output.write()``
+        """
+        self.output.write(to_bytes(data, self.encoding))
+
+
+    def _write_str(self, data):
+        """
+        Converts the given data then writes it
+
         :param data: Data to be written
         :return: The result of ``self.output.write()``
         """
@@ -191,18 +225,18 @@ class IOHandler(io.RawIOBase):
         """
         if line is None:
             # Empty line
-            line = '\n'
+            self.write('\n')
 
         else:
             # Format the line
             line = line.format(*args, **kwargs)
 
-        # Add the trailing new line
-        if line[-1] != '\n':
-            line += '\n'
+            # Write it
+            self.write(line)
+            if line[-1] != '\n':
+                # Add the trailing new line
+                self.write('\n')
 
-        # Write it
-        self.write(line)
         self.flush()
 
 
@@ -248,47 +282,66 @@ class ShellUtils(object):
         return states.get(state, "Unknown state (%d)".format(state))
 
 
-    def make_table(self, headers, lines):
+    def make_table(self, headers, lines, prefix=None):
         """
         Generates an ASCII table according to the given headers and lines
 
         :param headers: List of table headers (N-tuple)
-        :param lines: List of table lines (N-tuples).
+        :param lines: List of table lines (N-tuples)
+        :param prefix: Optional prefix for each line
         :return: The ASCII representation of the table
         :raise ValueError: Different number of columns between headers and lines
         """
-        if lines and len(headers) != len(lines[0]):
-            raise ValueError("Different sizes for header and lines")
+        # Normalize the prefix
+        prefix = str(prefix or "")
 
         # Maximum lengths
         lengths = [len(title) for title in headers]
 
+        # Store the number of columns (0-based)
+        nb_columns = len(lengths) - 1
+
         # Lines
         str_lines = []
-        for line in lines:
+        for idx, line in enumerate(lines):
             # Recompute lengths
-            i = 0
             str_line = []
             str_lines.append(str_line)
-            for entry in line:
-                str_entry = str(entry)
-                str_line.append(str_entry)
+            column = -1
 
-                if len(str_entry) > lengths[i]:
-                    lengths[i] = len(str_entry)
-                i += 1
+            try:
+                for column, entry in enumerate(line):
+                    str_entry = str(entry)
+                    str_line.append(str_entry)
+
+                    if len(str_entry) > lengths[column]:
+                        lengths[column] = len(str_entry)
+
+            except IndexError:
+                # Line too small/big
+                raise ValueError("Different sizes for header and lines "
+                                 "(line {0})".format(idx + 1))
+
+            except (TypeError, AttributeError):
+                # Invalid type of line
+                raise ValueError("Invalid type of line: %s",
+                                 type(line).__name__)
+
+            else:
+                if column != nb_columns:
+                    # Check if all lines have the same number of columns
+                    raise ValueError("Different sizes for header and lines "
+                                     "(line {0})".format(idx + 1))
 
         # Prepare the head (centered text)
-        format_str = "|"
-        i = 0
-        for length in lengths:
-            format_str += " {%d:^%d} |" % (i, length)
-            i += 1
+        format_str = "{0}|".format(prefix)
+        for column, length in enumerate(lengths):
+            format_str += " {%d:^%d} |" % (column, length)
 
         head_str = format_str.format(*headers)
 
         # Prepare the separator, according the length of the headers string
-        separator = '-' * len(head_str)
+        separator = '{0}{1}'.format(prefix, '-' * (len(head_str) - len(prefix)))
         idx = head_str.find('|')
         while idx != -1:
             separator = '+'.join((separator[:idx], separator[idx + 1:]))
@@ -370,7 +423,7 @@ class Shell(object):
         """
         Called if a command service has been found.
         Registers the methods of this service.
-        
+
         :param svc_ref: A reference to the found service
         :return: True if the commands have been registered
         """
@@ -385,35 +438,10 @@ class Shell(object):
         namespace = handler.get_namespace()
         commands = []
 
-        # Get the host exporting the service, if any
-        remote_host = None
-        if svc_ref.get_property("service.imported"):
-            # Imported service: store its host
-            remote_host = svc_ref.get_property("service.imported.from")
-
-        if not remote_host:
-            # Local service: register all its methods directly
-            for command, method in handler.get_methods():
-                self.register_command(namespace, command, method)
-                commands.append(command)
-
-        else:
-            # Imported service
-            _logger.info("Bound to a remote command handler from %s",
-                         remote_host)
-
-            # Prefix its name space
-            namespace = ".".join((remote_host, namespace))
-            for command, method_name in handler.get_methods_names():
-                # Use a proxy to call the methods
-                def proxy(*args, **kwargs):
-                    """
-                    Remote command proxy
-                    """
-                    return getattr(handler, method_name)(*args, **kwargs)
-
-                self.register_command(namespace, command, proxy)
-                commands.append(command)
+        # Register all service methods directly
+        for command, method in handler.get_methods():
+            self.register_command(namespace, command, method)
+            commands.append(command)
 
         # Store the reference
         self._bound_references[svc_ref] = handler
@@ -425,7 +453,7 @@ class Shell(object):
         """
         Called if a command service is gone.
         Unregisters its commands.
-        
+
         :param svc_ref: A reference to the unbound service
         :return: True if the commands have been unregistered
         """
@@ -457,6 +485,14 @@ class Shell(object):
         :return: True if the method has been registered, False if it was already
                  known or invalid
         """
+        if method is None:
+            _logger.error("No method given for %s.%s", namespace, command)
+            return False
+
+        # Store everything in lower case
+        namespace = (namespace or "").strip().lower()
+        command = (command or "").strip().lower()
+
         if not namespace:
             namespace = DEFAULT_NAMESPACE
 
@@ -464,14 +500,6 @@ class Shell(object):
             _logger.error("No command name given")
             return False
 
-        if method is None:
-            _logger.error("No method given for %s.%s", namespace, command)
-            return False
-
-        # Store everything in lower case
-        namespace = namespace.lower()
-        command = command.lower()
-
         if namespace not in self._commands:
             space = self._commands[namespace] = {}
         else:
@@ -498,7 +526,7 @@ class Shell(object):
         if not namespace:
             namespace = DEFAULT_NAMESPACE
 
-        namespace = namespace.lower()
+        namespace = namespace.strip().lower()
 
         if namespace not in self._commands:
             _logger.warning("Unknown name space: %s", namespace)
@@ -506,6 +534,7 @@ class Shell(object):
 
         if command is not None:
             # Remove the command
+            command = command.strip().lower()
             if command not in self._commands[namespace]:
                 _logger.warning("Unknown command: %s.%s", namespace, command)
                 return False
@@ -529,7 +558,7 @@ class Shell(object):
         If the command exists in the default name space, the returned list will
         only contain the default name space.
         Returns an empty list of the command is unknown
-        
+
         :param command: A command name
         :return: A list of name spaces
         """
@@ -551,7 +580,7 @@ class Shell(object):
         """
         Retrieves the name space and the command associated to the given
         command name.
-        
+
         :param cmd_name: The given command name
         :return: A 2-tuple (name space, command)
         :raise ValueError: Unknown command name
@@ -622,7 +651,9 @@ class Shell(object):
 
         # Execute it
         try:
-            return method(io_handler, *args, **kwargs)
+            result = method(io_handler, *args, **kwargs)
+            # None is considered as a success
+            return result is None or result
 
         except TypeError as ex:
             # Invalid arguments...
@@ -653,7 +684,7 @@ class Shell(object):
     def get_namespaces(self):
         """
         Retrieves the list of known name spaces (without the default one)
-        
+
         :return: The list of known name spaces
         """
         namespaces = list(self._commands.keys())
@@ -666,7 +697,7 @@ class Shell(object):
         """
         Retrieves the commands of the given name space. If *namespace* is None
         or empty, it retrieves the commands of the default name space
-        
+
         :param namespace: The commands name space
         :return: A list of commands names
         """
@@ -674,20 +705,43 @@ class Shell(object):
             # Default name space:
             namespace = DEFAULT_NAMESPACE
 
-        commands = list(self._commands[namespace].keys())
-        commands.sort()
-        return commands
+        try:
+            namespace.strip().lower()
+            commands = list(self._commands[namespace].keys())
+            commands.sort()
+            return commands
+
+        except KeyError:
+            # Unknown name space
+            return []
 
 
     def bundle_details(self, io_handler, bundle_id):
         """
-        Prints the details of the given bundle
+        Prints the details of the bundle with the given ID or name
         """
-        try:
-            bundle_id = int(bundle_id)
-            bundle = self._context.get_bundle(bundle_id)
+        bundle = None
 
-        except pelix.BundleException:
+        try:
+            # Convert the given ID into an integer
+            bundle_id = int(bundle_id)
+
+        except ValueError:
+            # Not an integer, suppose it's a bundle name
+            for bundle in self._context.get_bundles():
+                if bundle.get_symbolic_name() == bundle_id:
+                    break
+
+        else:
+            # Integer ID: direct access
+            try:
+                bundle = self._context.get_bundle(bundle_id)
+
+            except constants.BundleException:
+                pass
+
+        if bundle is None:
+            # No matching bundle
             io_handler.write_line("Unknown bundle ID: {0}", bundle_id)
             return
 
@@ -708,7 +762,7 @@ class Shell(object):
             else:
                 lines.append("\tn/a")
 
-        except pelix.BundleException as ex:
+        except constants.BundleException as ex:
             # Bundle in a invalid state
             lines.append("\tError: {0}".format(ex))
 
@@ -722,7 +776,7 @@ class Shell(object):
             else:
                 lines.append("\tn/a")
 
-        except pelix.BundleException as ex:
+        except constants.BundleException as ex:
             # Bundle in a invalid state
             lines.append("\tError: {0}".format(ex))
 
@@ -744,26 +798,24 @@ class Shell(object):
         bundles.insert(0, self._context.get_bundle(0))
 
         # Make the entries
-        lines = []
-        for bundle in bundles:
-            # Make the line
-            line = [str(entry) for entry in
-                    (bundle.get_bundle_id(), bundle.get_symbolic_name(),
-                    self._utils.bundlestate_to_str(bundle.get_state()),
-                    bundle.get_version())]
-
-            lines.append(line)
+        lines = [[str(entry) for entry in
+                        (bundle.get_bundle_id(),
+                         bundle.get_symbolic_name(),
+                         self._utils.bundlestate_to_str(bundle.get_state()),
+                         bundle.get_version())]
+                 for bundle in bundles]
 
         # Print'em all
         io_handler.write(self._utils.make_table(headers, lines))
+        io_handler.write_line("{0} bundles installed", len(lines))
 
 
     def service_details(self, io_handler, service_id):
         """
-        Prints the details of the given service
+        Prints the details of the service with the given ID
         """
         svc_ref = self._context.get_service_reference(None,
-                                '({0}={1})'.format(pelix.SERVICE_ID,
+                                '({0}={1})'.format(constants.SERVICE_ID,
                                                    service_id))
         if svc_ref is None:
             io_handler.write_line('Service not found: {0}', service_id)
@@ -771,11 +823,11 @@ class Shell(object):
 
         lines = []
         lines.append("ID    : {0}".format(svc_ref.get_property(
-                                            pelix.SERVICE_ID)))
+                                            constants.SERVICE_ID)))
         lines.append("Rank  : {0}".format(svc_ref.get_property(
-                                            pelix.SERVICE_RANKING)))
+                                            constants.SERVICE_RANKING)))
         lines.append("Specs : {0}".format(svc_ref.get_property(
-                                            pelix.OBJECTCLASS)))
+                                            constants.OBJECTCLASS)))
         lines.append("Bundle: {0}".format(svc_ref.get_bundle()))
         lines.append("Properties:")
         for key, value in svc_ref.get_properties().items():
@@ -802,34 +854,90 @@ class Shell(object):
         # Use the reverse order (ascending service IDs instead of descending)
         references.reverse()
 
-        lines = []
-        for ref in references:
-            # Make the line
-            line = [str(entry) for entry in
-                    (ref.get_property(pelix.SERVICE_ID),
-                     ref.get_property(pelix.OBJECTCLASS),
-                     ref.get_bundle(),
-                     ref.get_property(pelix.SERVICE_RANKING))]
-
-            lines.append(line)
+        # Construct the list of services
+        lines = [[str(entry) for entry in
+                        (ref.get_property(constants.SERVICE_ID),
+                         ref.get_property(constants.OBJECTCLASS),
+                         ref.get_bundle(),
+                         ref.get_property(constants.SERVICE_RANKING))]
+                 for ref in references]
 
         # Print'em all
         io_handler.write(self._utils.make_table(headers, lines))
+        io_handler.write_line("{0} services registered", len(lines))
 
 
     def __extract_help(self, method):
         """
-        Extract the help string from the given method
+        Formats the help string for the given method
+
+        :param method: The method to document
+        :return: A tuple: (arguments list, documentation line)
         """
         if method is None:
             return "(No associated method)"
 
-        return getattr(method, '__doc__', None) or "(Documentation missing)"
+        # Get the arguments
+        argspec = inspect.getargspec(method)
+
+        # Compute the number of arguments with default value
+        if argspec.defaults is not None:
+            nb_optional = len(argspec.defaults)
+
+            # Let the mandatory arguments as they are
+            args = ["<{0}>".format(arg) for arg in argspec.args[2:-nb_optional]]
+
+            # Add the other arguments
+            for name, value in zip(argspec.args[-nb_optional:],
+                                   argspec.defaults[-nb_optional:]):
+                if value is not None:
+                    args.append('[<{0}>={1}]'.format(name, value))
+
+                else:
+                    args.append('[<{0}>]'.format(name))
+
+        else:
+            # All arguments are mandatory
+            args = ["<{0}>".format(arg) for arg in argspec.args[2:]]
+
+        # Extra arguments
+        if argspec.keywords:
+            args.append('[<property=value> ...]')
+
+        if argspec.varargs:
+            args.append("...")
+
+        # Get the documentation string
+        doc = inspect.getdoc(method) or "(Documentation missing)"
+
+        return ' '.join(args), ' '.join(doc.split())
+
+
+    def __print_command_help(self, io_handler, namespace, cmd_name):
+        """
+        Prints the documentation of the given command
+
+        :param io_handler: I/O handler
+        :param namespace: Name space of the command
+        :param cmd_name: Name of the command
+        """
+        # Extract documentation
+        args, doc = self.__extract_help(self._commands[namespace][cmd_name])
+
+        # Print the command name, and its arguments
+        if args:
+            io_handler.write_line("- {0} {1}", cmd_name, args)
+        else:
+            io_handler.write_line("- {0}", cmd_name)
+
+        # Print the documentation line
+        io_handler.write_line("\t\t{0}", doc)
 
 
     def print_help(self, io_handler, command=None):
         """
-        Prints the available methods and their documentation
+        Prints the available methods and their documentation, or the
+        documentation of the given command.
         """
         if command:
             # Single command mode
@@ -845,9 +953,7 @@ class Shell(object):
             else:
                 # Print the help
                 io_handler.write_line("* Name space '{0}':", namespace)
-                io_handler.write_line("- {0}", cmd_name)
-                doc = self.__extract_help(self._commands[namespace][cmd_name])
-                io_handler.write_line("\t\t{0}", ' '.join(doc.split()))
+                self.__print_command_help(io_handler, namespace, cmd_name)
 
         else:
             # Get all name spaces
@@ -863,10 +969,8 @@ class Shell(object):
                 names = [command for command in self._commands[namespace]]
                 names.sort()
 
-                for name in names:
-                    io_handler.write_line("- {0}", name)
-                    doc = self.__extract_help(self._commands[namespace][name])
-                    io_handler.write_line("\t\t{0}", ' '.join(doc.split()))
+                for cmd_name in names:
+                    self.__print_command_help(io_handler, namespace, cmd_name)
 
 
     def properties_list(self, io_handler):
@@ -891,7 +995,7 @@ class Shell(object):
 
     def property_value(self, io_handler, name):
         """
-        property <name> - Prints the value of the given property, looking into
+        Prints the value of the given property, looking into
         framework properties then environment variables.
         """
         io_handler.write_line(str(self._context.get_property(name)))
@@ -916,7 +1020,7 @@ class Shell(object):
 
     def environment_value(self, io_handler, name):
         """
-        sysprop <name> - Prints the value of the given environment variable
+        Prints the value of the given environment variable
         """
         io_handler.write_line(os.getenv(name))
 
@@ -928,6 +1032,9 @@ class Shell(object):
         # Extract frames
         frames = sys._current_frames()
 
+        # Get the thread ID -> Thread mapping
+        names = threading._active.copy()
+
         # Sort by thread ID
         thread_ids = list(frames.keys())
         thread_ids.sort()
@@ -937,11 +1044,18 @@ class Shell(object):
             # Get the corresponding stack
             stack = frames[thread_id]
 
+            # Try to get the thread name
+            try:
+                name = names[thread_id].name
+
+            except KeyError:
+                name = "<unknown>"
+
             # Construct the code position
-            lines.append('Thread ID: {0}'.format(thread_id))
-            lines.append('Stack trace:')
+            lines.append('Thread ID: {0} - Name: {1}'.format(thread_id, name))
+            lines.append('Line:')
             lines.extend((line.rstrip()
-                          for line in traceback.format_stack(stack, 8)))
+                          for line in traceback.format_stack(stack, 1)))
             lines.append('')
 
         lines.append('')
@@ -952,10 +1066,12 @@ class Shell(object):
 
     def thread_details(self, io_handler, thread_id):
         """
-        thread <id> - Prints details about the given thread
+        Prints details about the thread with the given ID (not its name)
         """
         try:
-            stack = sys._current_frames()[int(thread_id)]
+            # Get the stack
+            thread_id = int(thread_id)
+            stack = sys._current_frames()[thread_id]
 
         except KeyError:
             io_handler.write_line("Unknown thread ID: {0}", thread_id)
@@ -964,8 +1080,15 @@ class Shell(object):
             io_handler.write_line("Invalid thread ID: {0}", thread_id)
 
         else:
+            # Get the name
+            try:
+                name = threading._active[thread_id].name
+
+            except KeyError:
+                name = "<unknown>"
+
             lines = []
-            lines.append('Thread ID: {0}'.format(thread_id))
+            lines.append('Thread ID: {0} - Name: {1}'.format(thread_id, name))
             lines.append('Stack trace:')
             lines.extend((line.rstrip()
                           for line in traceback.format_stack(stack)))
@@ -983,7 +1106,7 @@ class Shell(object):
 
     def start(self, io_handler, bundle_id):
         """
-        start <bundle_id> - Starts the given bundle ID
+        Starts the bundle with the given ID
         """
         bundle_id = int(bundle_id)
         bundle = self._context.get_bundle(bundle_id)
@@ -995,7 +1118,7 @@ class Shell(object):
 
     def stop(self, io_handler, bundle_id):
         """
-        stop <bundle_id> - Stops the given bundle
+        Stops the bundle with the given ID
         """
         bundle_id = int(bundle_id)
         bundle = self._context.get_bundle(bundle_id)
@@ -1007,7 +1130,7 @@ class Shell(object):
 
     def update(self, io_handler, bundle_id):
         """
-        update <bundle_id> - Updates the given bundle ID
+        Updates the bundle with the given ID
         """
         bundle_id = int(bundle_id)
         bundle = self._context.get_bundle(bundle_id)
@@ -1017,17 +1140,17 @@ class Shell(object):
         bundle.update()
 
 
-    def install(self, io_handler, location):
+    def install(self, io_handler, module_name):
         """
-        install <bundle_id> - Installs the given bundle
+        Installs the bundle with the given module name
         """
-        bundle = self._context.install_bundle(location)
+        bundle = self._context.install_bundle(module_name)
         io_handler.write_line("Bundle ID: {0}", bundle.get_bundle_id())
 
 
     def uninstall(self, io_handler, bundle_id):
         """
-        uninstall <bundle_id> - Uninstalls the given bundle
+        Uninstalls the bundle with the given ID
         """
         bundle_id = int(bundle_id)
         bundle = self._context.get_bundle(bundle_id)
@@ -1087,9 +1210,7 @@ class PelixActivator(object):
                                                        utils, {})
 
             # Register the service listener
-            spec_filter = '({0}={1})'.format(pelix.OBJECTCLASS,
-                                             SHELL_COMMAND_SPEC)
-            context.add_service_listener(self, spec_filter)
+            context.add_service_listener(self, None, SHELL_COMMAND_SPEC)
 
             # Register existing command services
             refs = context.get_all_service_references(SHELL_COMMAND_SPEC,
@@ -1100,7 +1221,7 @@ class PelixActivator(object):
 
             _logger.info("Shell services registered")
 
-        except pelix.BundleException as ex:
+        except constants.BundleException as ex:
             _logger.exception("Error registering the shell service: %s", ex)
 
 
