@@ -8,7 +8,7 @@ Provides commands to the Pelix shell to work with the EventAdmin service
 :author: Thomas Calmant
 :copyright: Copyright 2013, isandlaTech
 :license: GPLv3
-:version: 0.1
+:version: 0.1.2
 :status: Alpha
 
 ..
@@ -30,8 +30,8 @@ Provides commands to the Pelix shell to work with the EventAdmin service
 """
 
 # Module version
-__version_info__ = (0, 1, 1)
-__version__ = ".".join(map(str, __version_info__))
+__version_info__ = (0, 1, 2)
+__version__ = ".".join(str(x) for x in __version_info__)
 
 # Documentation strings format
 __docformat__ = "restructuredtext en"
@@ -79,24 +79,15 @@ class EventAdminCommands(object):
                 ("post", self.post)]
 
 
-    def get_methods_names(self):
-        """
-        Retrieves the list of tuples (command, method name) for this command
-        handler.
-        """
-        return [(command, method.__name__)
-                for command, method in self.get_methods()]
-
-
     def send(self, io_handler, topic, **kwargs):
         """
-        send <topic> [<property=value>] - Sends an event (blocking)
+        Sends an event (blocking)
         """
         self._events.send(topic, kwargs)
 
 
     def post(self, io_handler, topic, **kwargs):
         """
-        post <topic> [<property=value>] - Posts an event (asynchronous)
+        Posts an event (asynchronous)
         """
         self._events.post(topic, kwargs)
