@@ -8,7 +8,7 @@ Provides commands to the Pelix shell to get the state of iPOPO instances.
 :author: Thomas Calmant
 :copyright: Copyright 2013, isandlaTech
 :license: GPLv3
-:version: 0.2
+:version: 0.2.2
 :status: Alpha
 
 ..
@@ -30,8 +30,8 @@ Provides commands to the Pelix shell to get the state of iPOPO instances.
 """
 
 # Module version
-__version_info__ = (0, 2, 1)
-__version__ = ".".join(map(str, __version_info__))
+__version_info__ = (0, 2, 2)
+__version__ = ".".join(str(x) for x in __version_info__)
 
 # Documentation strings format
 __docformat__ = "restructuredtext en"
@@ -58,7 +58,7 @@ _logger = logging.getLogger(__name__)
 def ipopo_state_to_str(state):
     """
     Converts the state of a component instance to its string representation
-    
+
     :param state: The state of an iPOPO component
     :return: A string representation of the state
     """
@@ -109,18 +109,6 @@ class IPopoCommands(object):
                 ]
 
 
-    def get_methods_names(self):
-        """
-        Retrieves the list of tuples (command, method name) for this command
-        handler.
-        """
-        result = []
-        for command, method in self.get_methods():
-            result.append((command, method.__name__))
-
-        return result
-
-
     def list_factories(self, io_handler):
         """
         Lists the available iPOPO component factories
@@ -146,7 +134,7 @@ class IPopoCommands(object):
 
     def factory_details(self, io_handler, name):
         """
-        factory <name> - Prints the details of the given component factory
+        Prints the details of the given component factory
         """
         try:
             details = self._ipopo.get_factory_details(name)
@@ -190,7 +178,7 @@ class IPopoCommands(object):
 
     def instance_details(self, io_handler, name):
         """
-        instance <name> - Prints the details of the given component instance
+        Prints the details of the given component instance
         """
         try:
             details = self._ipopo.get_instance_details(name)
@@ -200,35 +188,49 @@ class IPopoCommands(object):
                                   name, ex)
             return
 
+        # Basic information
         lines = []
-        lines.append("Name   : {0}".format(details["name"]))
-        lines.append("Factory: {0}".format(details["factory"]))
-        lines.append("State  : {0}".format(ipopo_state_to_str(
-                                                        details["state"])))
-        if "service" in details:
-            lines.append("Service: {0}".format(details["service"]))
+        lines.append("Name     : {0}".format(details["name"]))
+        lines.append("Factory  : {0}".format(details["factory"]))
+        lines.append("Bundle ID: {0}".format(details["bundle_id"]))
+        lines.append("State    : {0}".format(
+                                         ipopo_state_to_str(details["state"])))
 
+        # Provided services
+        lines.append("Services :")
+        lines.extend("\t{0}".format(svc_reference)
+                     for svc_reference in details["services"].values())
+
+        # Requirements
         lines.append("Dependencies:")
         for field, infos in details["dependencies"].items():
             lines.append("\tField: {0}".format(field))
-            lines.append("\t\tOptional : {0}".format(infos["optional"]))
-            lines.append("\t\tAggregate: {0}".format(infos["aggregate"]))
+            lines.append("\t\tSpecification: {0}"\
+                         .format(infos['specification']))
             if "filter" in infos:
                 lines.append("\t\tFilter   : {0}".format(infos["filter"]))
+            lines.append("\t\tOptional : {0}".format(infos["optional"]))
+            lines.append("\t\tAggregate: {0}".format(infos["aggregate"]))
 
             lines.append("\t\tHandler  : {0}".format(infos["handler"]))
             lines.append("\t\tBindings :")
             for ref in infos["bindings"]:
                 lines.append('\t\t\t{0}'.format(ref))
 
+        # Properties
+        lines.append("Properties:")
+        lines.append(self._utils.make_table(("Key", "Value"),
+                                            details['properties'].items(),
+                                            "\t"))
+
         lines.append("")
         io_handler.write('\n'.join(lines))
 
 
     def instantiate(self, io_handler, factory, name, **kwargs):
         """
-        instantiate <factory> <name> [<property=value> ...] - Instantiate a
-        component of the given factory with the given name and properties
+        Instantiates a component of the given factory with the given name and
+        properties
         """
         try:
             self._ipopo.instantiate(factory, name, kwargs)
@@ -246,7 +248,7 @@ class IPopoCommands(object):
 
     def kill(self, io_handler, name):
         """
-        kill <name> - Kills the given component instance
+        Kills the given component instance
         """
         try:
             self._ipopo.kill(name)
