@@ -9,7 +9,7 @@ telnet or netcat.
 :author: Thomas Calmant
 :copyright: Copyright 2013, isandlaTech
 :license: GPLv3
-:version: 0.1.1
+:version: 0.1.2
 :status: Alpha
 
 ..
@@ -31,30 +31,33 @@ telnet or netcat.
 """
 
 # Module version
-__version_info__ = (0, 1, 1)
-__version__ = ".".join(map(str, __version_info__))
+__version_info__ = (0, 1, 2)
+__version__ = ".".join(str(x) for x in __version_info__)
 
 # Documentation strings format
 __docformat__ = "restructuredtext en"
 
 # ------------------------------------------------------------------------------
 
+# iPOPO decorators
 from pelix.ipopo.decorators import ComponentFactory, Requires, Property, \
     Validate, Invalidate, Provides
-from pelix.shell import SHELL_SERVICE_SPEC, REMOTE_SHELL_SPEC
 
-# ------------------------------------------------------------------------------
+# Shell constants
+import pelix.shell
 
+# Standard library
 from select import select
-
 import logging
-import sys
 import threading
 import socket
 
-if sys.version_info[0] >= 3:
+try:
+    # Python 3
     import socketserver
-else:
+
+except ImportError:
+    # Python 2
     import SocketServer as socketserver
 
 # ------------------------------------------------------------------------------
@@ -99,7 +102,7 @@ class RemoteConsole(socketserver.StreamRequestHandler):
     def __init__(self, shell_svc, active_flag, *args):
         """
         Sets up members
-        
+
         :param shell_svc: The underlying Pelix shell service
         :param active_flag: Common flag for stopping the client communication
         """
@@ -111,7 +114,7 @@ class RemoteConsole(socketserver.StreamRequestHandler):
     def send(self, data):
         """
         Tries to send data to the client.
-        
+
         :param data: Data to be sent
         :return: True if the data was sent, False on error
         """
@@ -219,10 +222,10 @@ class ThreadingTCPServerFamily(socketserver.ThreadingTCPServer):
             # Explicitly ask to be accessible both by IPv4 and IPv6
             # Some versions of Python don't have V6ONLY.
             # On Linux, IPC6_V6ONLY = 26
-            IPV6_V6ONLY = getattr(socket, "IPV6_V6ONLY", 26)
+            opt_ipv6_only = getattr(socket, "IPV6_V6ONLY", 26)
 
             try:
-                self.socket.setsockopt(socket.IPPROTO_IPV6, IPV6_V6ONLY, 0)
+                self.socket.setsockopt(socket.IPPROTO_IPV6, opt_ipv6_only, 0)
 
             except socket.error as ex:
                 # Log the error
@@ -232,7 +235,7 @@ class ThreadingTCPServerFamily(socketserver.ThreadingTCPServer):
 def _create_server(shell, server_address, port):
     """
     Creates the TCP console on the given address and port
-    
+
     :param shell: The remote shell handler
     :param server_address: Server bound address
     :param port: Server port
@@ -262,11 +265,11 @@ def _create_server(shell, server_address, port):
 
 # ------------------------------------------------------------------------------
 
-@ComponentFactory("ipopo-remote-shell-factory")
-@Provides(REMOTE_SHELL_SPEC)
+@ComponentFactory(pelix.shell.FACTORY_REMOTE_SHELL)
+@Provides(pelix.shell.REMOTE_SHELL_SPEC)
 @Property("_address", "pelix.shell.address", "localhost")
 @Property("_port", "pelix.shell.port", 9000)
-@Requires("_shell", SHELL_SERVICE_SPEC)
+@Requires("_shell", pelix.shell.SHELL_SERVICE_SPEC)
 class IPopoRemoteShell(object):
     """
     The iPOPO Remote Shell, based on the Pelix Shell
@@ -289,7 +292,7 @@ class IPopoRemoteShell(object):
     def get_access(self):
         """
         Implementation of the remote shell specification
-        
+
         :return: A (host, port) tuple
         """
         return (self._address, self._port)
@@ -298,7 +301,7 @@ class IPopoRemoteShell(object):
     def get_banner(self):
         """
         Retrieves the shell banner
-        
+
         :return: The shell banner
         """
         line = '-' * 72
@@ -311,7 +314,7 @@ class IPopoRemoteShell(object):
     def get_ps1(self):
         """
         Returns the shell prompt
-        
+
         :return: The shell prompt
         """
         return self._shell.get_ps1()
@@ -320,9 +323,9 @@ class IPopoRemoteShell(object):
     def handle_line(self, rfile, wfile, line):
         """
         Handles the command line.
-        
+
         **Does not catch exceptions !**
-        
+
         :param rfile: Input file-like object
         :param wfile: Output file-like object
         :param line: The command line
